<?php

namespace Leenooks\Providers;

use Illuminate\Support\Facades\Blade;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\ServiceProvider;

class CustomBladeServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {
		Blade::directive('css',function(string $expression) {
			return $this->resolve('css',$expression);
		});

		Blade::directive('js',function($expression) {
			return $this->resolve('js',$expression);
		});
    }

	private function resolve(string $content,string $expression): string
	{
		if (str_contains($expression,',')) {
			[$type,$arguments] = explode(',',$expression,2);
			$arguments = explode('|',$arguments);

		} else {
			$type = $expression;
			$arguments = [];
		}

		$return = collect();
		$css = collect();
		$js = collect();

		switch ($type) {
			case 'datatables':
			case 'select2':
			case 'simplemde':
				Log::alert(sprintf('Blade @css/@js for %s deprecicated, use @pa',$type));
				break;

			case 'datepick':
				switch ($content) {
					case 'css':
						$css->put($type,'https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.9.0/css/bootstrap-datepicker.min.css');
						break;

					case 'js':
						$js->put($type,'https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.9.0/js/bootstrap-datepicker.min.js');
				}

				break;

			case 'debounce':
				switch ($content) {
					case 'js':
						$js->put($type,'js/debounce.js');
				}

				break;

			case 'highcharts':
				switch ($content) {
					case 'js':
						// Base
						$js->put($type,'https://code.highcharts.com/highcharts.js');

						foreach ($arguments as $option) {
							$key = $type.':'.$option;
							switch ($option) {
								case '3d':
									$js->put($key,'https://code.highcharts.com/highcharts-3d.js');
									$js->put($key.'mouseover','/plugin/highcharts/3dmouseover.js');
									break;

								case 'data':
									$js->put($key,'https://code.highcharts.com/modules/data.js');
									break;

								case 'defaults':
									$js->put($key,'/plugin/highcharts/defaults.js');
									break;

								case 'drilldown':
									$js->put($key,'https://code.highcharts.com/modules/drilldown.js');
									break;

								case 'export':
									$js->put($key,'https://code.highcharts.com/modules/exporting.js');
									$js->put($key.'data','https://code.highcharts.com/modules/export-data.js');
									break;

								case 'funnel':
									$js->put($key,'https://code.highcharts.com/modules/funnel.js');
									break;

								case 'heatmap':
									$js->put($key,'https://code.highcharts.com/modules/heatmap.js');
									break;

								case 'solidguage':
									$js->put($key,'https://code.highcharts.com/highcharts-more.js');
									$js->put($key.'accessiblity','https://code.highcharts.com/modules/accessibility.js');
									$js->put($key.'solid-guage','https://code.highcharts.com/modules/solid-gauge.js');
									break;

								case 'theme-dark':
									$js->put($key,'https://code.highcharts.com/themes/dark-unica.js');
									break;

								default:
									throw new \Exception(sprintf('Unknown [%s] option: [%s:%s]',$type,$content,$option));
							}
						}
				}

				break;

			case 'highcharts-stock':
				switch ($content) {
					case 'js':
						// Base
						$js->put($type,'https://code.highcharts.com/stock/highstock.js');

						foreach ($arguments as $option) {
							$key = $type.':'.$option;
							switch ($option) {
								case 'data':
									$js->put($key,'https://code.highcharts.com/stock/modules/data.js');
									break;

								case 'export':
									$js->put($key,'https://code.highcharts.com/stock/modules/exporting.js');
									break;

								default:
									throw new \Exception(sprintf('Unknown [%s] option: [%s:%s]',$type,$content,$option));
							}
						}
				}

				break;

			case 'summernote':
				switch ($content) {
					case 'css':
						// Base
						$css->put($type,'https://cdnjs.cloudflare.com/ajax/libs/summernote/0.8.18/summernote-bs4.css');

						break;

					case 'js':
						// Base
						$js->put($type,'https://cdnjs.cloudflare.com/ajax/libs/summernote/0.8.18/summernote-bs4.js');
				}

				break;

			case 'validation':
				switch ($content) {
					case 'js':
						// Base
						$js->put('validate','https://cdn.jsdelivr.net/npm/jquery-validation@1.19.3/dist/jquery.validate.min.js');
						$js->put('validate.additional','https://cdn.jsdelivr.net/npm/jquery-validation@1.19.3/dist/additional-methods.min.js');
				}

				break;

			default:
				throw new \Exception(sprintf('Unknown Expression: [%s]',$expression));
		}

		if ($js->count())
			$return = $return->merge($js->map(fn($item)=>sprintf('<script type="text/javascript" src="%s"></script>',$item)));

		if ($css->count())
			$return = $return->merge($css->map(fn($item)=>sprintf('<link rel="stylesheet" href="%s">',$item)));

		return $return->join('');
	}
}